<?php

declare(strict_types=1);

namespace Drupal\media_acquiadam\Form;

use Drupal\acquia_dam\Entity\MediaSourceField;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the settings form to migrate Acquia DAM integration from a legacy to a more modern solution.
 */
class AcquiadamMigration extends ConfigFormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The media source plugin manager service.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $mediaSourcePluginManager;

  /**
   * The migration service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Store module-wide used constant as class attribute.
   *
   * @var string
   */
  public const MODULE_SETTINGS_NAME = 'media_acquiadam.migration_preferences';

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->moduleHandler = $container->get('module_handler');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->mediaSourcePluginManager = $container->get('plugin.manager.media.source');
    $instance->configFactory = $container->get('config.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_acquiadam_migration';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      self::MODULE_SETTINGS_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // Bail out when `acquia_dam` module necessary to setup migration is not installed by any weird reason.
    if (!$this->moduleHandler->moduleExists('acquia_dam')) {
      $this->messenger()->addError(t('The migration process requires that both the source <em>Media: Acquia DAM</em> (<code>media_acquiadam</code>) and the target <em>Acquia DAM</em> (<code>acquia_dam</code>) modules be installed on the website. <a href=":url">Enable the missing module</a>', [
        // We know from the module's routing file that the URL path of the current page is fixed.
        ':url' => Url::fromRoute('system.modules_list', [], ['query' => ['destination' => '/admin/config/media/acquiadam/migration']])->toString(),
      ]));
      unset($form['actions']);

      return $form;
    }

    $dam_bundles_all = [];
    foreach (['field_acquiadam_asset_id', MediaSourceField::SOURCE_FIELD_NAME] as $source_field) {
      $dam_bundles_all += $this->entityTypeManager->getStorage('media_type')->loadByProperties(['source_configuration.source_field' => $source_field]);
    }

    // Bail out when no media types exist at all.
    if (count($dam_bundles_all) === 0) {
      $this->messenger()->addError(t('No DAM-related media types exist on the site. The migration process handles only media types which are based on the source plugins provided by either the <em>Media: Acquia DAM</em> (<code>media_acquiadam</code>) or the <em>Acquia DAM</em> (<code>acquia_dam</code>) modules. <a href=":url">Uninstall the <em>Media: Acquia DAM</em> module</a>', [
        ':url' => Url::fromRoute('system.modules_uninstall')->toString(),
      ]));
      unset($form['actions']);

      return $form;
    }

    // Start building up the form.
    $form['introductional_guide'] = [
      '#type' => 'container',
      'children' => [
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('This page lists all the media types that are using any source plugin provided by either the <em>Media: Acquia DAM</em> (<code>media_acquiadam</code>) or the <em>Acquia DAM</em> (<code>acquia_dam</code>) modules. In most cases, this results in twelve rows: five default media types from the prior and seven from the latter module. However, any other manually created, custom media types using such source plugins are also listed. Media types of Drupal core\'s Media API or defined by any other contrib module do not appear here.'),
        ],
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('The first segment of the list displays the media types provided by the “legacy” module. Their human label can be renamed and their machine name is displayed only for informational purposes. If any media item exists on the site of a given type, then their deletion is disallowed. If a legacy media type does not contain any media item, then their deletion is recommended.'),
        ],
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('If a legacy media type is being kept (most probably because it contains media items), then choosing a file-handling method is mandatory. If the legacy media type\'s name suggests that it can contain “Audio” or “Generic” type of assets from Widen DAM, then the <em>Sync</em> method is suggested to apply. If you know about any other manually created, custom media types containing such assets too, then please ensure that you also opt for the <em>Sync</em> method for them as well.'),
        ],
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('Deciding about the target source plugin is also mandatory for all legacy media types not being deleted. The drop-down field lists source plugins provided by the modern module only. Some automated guesswork is also implemented here to offer suggestions but always pay attention to opting for the most appropriate source plugin for each legacy media type. If no match is found, then choose the “Generic” one.'),
        ],
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('The lower half of the table lists the media types provided by the modern module. Mostly the same terms apply to them except their file-handling and source plugin cannot be changed anymore.'),
        ],
      ],
    ];

    $form['bundle_list'] = [
      '#type' => 'table',
      '#header' => [
        'bundle_name' => $this->t('Media type label and name'),
        'item_count' => $this->t('Number of media items'),
        'separator' => '',
        'fetch_method' => $this->t('Method to handle media files'),
        'source_type' => $this->t('Target source type'),
        'deletion' => '',
      ],
      '#rows' => [],
      '#colgroups' => [
        [
          'data' => [
            [
              'span' => [5],
            ],
          ],
        ],
        [
          'data' => [
            [
              'style' => ['background-color: rgba(255, 0, 0, 0.15);'],
            ],
          ],
        ],
      ]
    ];

    $config = $this->configFactory->get(self::MODULE_SETTINGS_NAME)->get('crud_operations');

    foreach ($dam_bundles_all as $bundle_id => $bundle_entity) {
      $is_provided_by_target_module = ($bundle_entity->get('source_configuration')['source_field'] === MediaSourceField::SOURCE_FIELD_NAME);
      $jquery_selector = ':input[id="edit-bundle-list-' . str_replace('_', '-', $bundle_id) . '-deletion"]';
      $asset_kind_word = preg_replace('/acquia_dam_([^_]+)(?:_asset)?$/', '$1', $bundle_id, -1, $count);
      // Consistent naming patterns FTW!
      switch ($asset_kind_word) {
        case 'asset':
          $asset_kind_word = 'generic';
          break;
        case 'document':
          $asset_kind_word = 'documents';
          break;
      }
      if ($count < 1) {
        $asset_kind_word = 'unknown';
      }

      $all_bundle_dependencies = $bundle_entity->getDependencies();
      if (isset($all_bundle_dependencies['module'])) {
        unset($all_bundle_dependencies['module']);
      }
      $bundle_has_dependencies = (bool) count($all_bundle_dependencies);

      $form['bundle_list'][$bundle_id]['bundle_name'] = [
        '#type' => 'textfield',
        '#value' => $config['update'][$bundle_id]['new_label'] ?? $bundle_entity->label(),
        '#description' => $this->t('Machine name: <code>@media-type-machine-name</code>', [
          '@media-type-machine-name' => $bundle_id,
        ]),
        '#maxlength' => 128,
        '#size' => 20,
        '#states' => [
          'disabled' => [$jquery_selector => ['checked' => TRUE]],
          'optional' => [$jquery_selector => ['checked' => TRUE]],
        ],
        '#attributes' => [
          'placeholder' => $this->t('e.g. @suggested_media_bundle_label', [
            '@suggested_media_bundle_label' => ucwords(implode(' ', explode('_', $bundle_id))),
          ]),
        ],
      ];

      $item_count = count($this->entityTypeManager->getStorage('media')->loadByProperties(['bundle' => $bundle_id]));
      if ($item_count > 0) {
        $form['bundle_list'][$bundle_id]['item_count'] = [
          '#type' => 'html_tag',
          '#tag' => 'h1',
          '#value' => $item_count,
          '#wrapper_attributes' => [
            'style' => 'text-align: center;',
          ],
        ];
      }
      else {
        $form['bundle_list'][$bundle_id]['item_count'] = [
          '#markup' => $this->t('None'),
          '#wrapper_attributes' => [
            'style' => 'text-align: center;',
          ],
        ];
      }

      $form['bundle_list'][$bundle_id]['separator'] = [
        '#type' => 'html_tag',
        '#tag' => 'h1',
        '#value' => "\u{2794}",
        '#wrapper_attributes' => [
          'style' => 'text-align: center; color: rgba(0, 0, 0, 0.33);',
        ],
      ];

      if ($is_provided_by_target_module) {
        $form['bundle_list'][$bundle_id]['fetch_method'] = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => $this->t('Embed'),
          '#attributes' => [
            'title' => $this->t("This media type uses a source plugin provided by the target module, thus there's no reason to change its fetch method."),
            'style' => 'cursor: help;',
          ],
        ];

        // Override default value of render element when asset syncing is enabled.
        $bundle_source_configuration = $bundle_entity->get('source_configuration');
        if (isset($bundle_source_configuration['download_assets']) && $bundle_source_configuration['download_assets']) {
          $form['bundle_list'][$bundle_id]['fetch_method']['#value'] = $this->t('Sync');
        }
      }
      else {
        // Convert a boolean stored in config to strings of the default option.
        if (isset($config['update'][$bundle_id]['asset_download_enabled'])) {
          if ($config['update'][$bundle_id]['asset_download_enabled']) {
            $value_from_config = 'sync';
          }
          else {
            $value_from_config = 'embed';
          }
        }

        $form['bundle_list'][$bundle_id]['fetch_method'] = [
          '#type' => 'select',
          '#options' => [
            'embed' => $this->t('Embed'),
            'sync' => $this->t('Sync'),
          ],
          '#sort_options' => TRUE,
          '#empty_option' => $this->t('- Select -'),
          '#empty_value' => '',
          '#default_value' => $value_from_config ?? NULL,
          '#description' => '<span>&nbsp;</span>',
          '#states' => [
            'disabled' => [$jquery_selector => ['checked' => TRUE]],
            'optional' => [$jquery_selector => ['checked' => TRUE]],
          ],
        ];
        unset($value_from_config);

        // Suggesting a possibly viable option to choose.
        if (in_array($asset_kind_word, ['generic', 'audio'])) {
          $form['bundle_list'][$bundle_id]['fetch_method']['#description'] = $this->t('Suggestion: <em>Sync</em>');
          $form['bundle_list'][$bundle_id]['fetch_method']['#attributes'] = [
            'title' => $this->t('Assuming based on the media type machine name that it contains “%asset_kind_one_word” type of assets from Widen, it is recommended that the “Sync” fetch method should be applied due to the lack of support embedding such assets.', [
              '%asset_kind_one_word' => ucfirst($asset_kind_word),
            ]),
            'style' => 'cursor: help;',
          ];
        }
      }

      // Sort out possible target media source plugins to offer for selection.
      foreach ($this->mediaSourcePluginManager->getDefinitions() as $media_source_plugin_id => $media_source_plugin) {
        if ($media_source_plugin['provider'] === 'acquia_dam') {
          $plugin_options[$media_source_plugin_id] = $media_source_plugin['label'];
        }
      }

      if ($is_provided_by_target_module) {
        $form['bundle_list'][$bundle_id]['source_type'] = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => $plugin_options[$bundle_entity->get('source')],
          '#attributes' => [
            'title' => $this->t("This media type uses a source plugin provided by the target module, thus there's no reason to change its source type."),
            'style' => 'cursor: help;',
          ],
        ];
      }
      else {
        $form['bundle_list'][$bundle_id]['source_type'] = [
          '#type' => 'select',
          '#options' => $plugin_options,
          '#sort_options' => TRUE,
          '#empty_option' => $this->t('- Select -'),
          '#empty_value' => '',
          '#default_value' => $config['update'][$bundle_id]['source_plugin'] ?? NULL,
          '#description' => '<span>&nbsp;</span>',
          '#states' => [
            'disabled' => [$jquery_selector => ['checked' => TRUE]],
            'optional' => [$jquery_selector => ['checked' => TRUE]],
          ],
        ];

        // Suggesting a possibly viable option to choose.
        $plugin_name_to_suggest = 'acquia_dam_asset:' . $asset_kind_word;
        if (isset($plugin_options[$plugin_name_to_suggest])) {
          $form['bundle_list'][$bundle_id]['source_type']['#default_value'] = $plugin_name_to_suggest;
          $form['bundle_list'][$bundle_id]['source_type']['#attributes'] = [
            'title' => $this->t('Assuming based on the media type machine name that it contains “%asset_kind_one_word” type of assets from Widen, a recommended source plugin has been pre-selected.', [
              '%asset_kind_one_word' => ucfirst($asset_kind_word),
            ]),
            'style' => 'cursor: help;',
          ];
        }
      }

      $form['bundle_list'][$bundle_id]['deletion'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Delete'),
      ];

      // First, just softly suggest considering removing the clutter of unused media types.
      if (!$is_provided_by_target_module && $item_count < 1) {
        $form['bundle_list'][$bundle_id]['deletion']['#default_value'] = TRUE;
      }

      // Then taking into account if their decision has been already made.
      if (isset($config['delete']) && in_array($bundle_id, $config['delete'])) {
        $form['bundle_list'][$bundle_id]['deletion']['#default_value'] = TRUE;
      }

      // Finally, hard override their decision depending on if the current status has changed since the last time when
      // the config was saved. Some possible scenarios:
      //   - If any new media items have been created, so let's disable the checkbox.
      //   - If all media items have been deleted, so let's still mark the checkbox.

      if ($item_count >= 1 || $bundle_has_dependencies) {
        $form['bundle_list'][$bundle_id]['deletion']['#value'] = FALSE;
        $form['bundle_list'][$bundle_id]['deletion']['#disabled'] = TRUE;
        $form['bundle_list'][$bundle_id]['deletion']['#attributes'] = [
          'title' => $this->t('Media items exist of this type (or other dependencies detected), and thus this type cannot be deleted.'),
          'style' => 'cursor: help;',
        ];
      }
    }

    $form['form_submission_guide'] = [
      '#type' => 'container',
      'children' => [
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('Once the migration preferences have been set up above, then you might want to perform the described changes. Clicking the “Save & migrate” button first confirms your decision, then starts a batch process to process affected media types. Finally, it reports its results in the Watchdog.'),
        ],
        [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => $this->t('Alternatively, clicking the “Save” button simply stores the settings in configuration only. In this case, you can initiate the migration process via CLI by running the <code>$ drush acquiadam:migrate</code> command.'),
        ],
      ],
    ];

    $form['actions']['submit']['#value'] = $this->t('Save');
    unset($form['actions']['submit']['#button_type']);
    $form['actions']['save_and_migrate'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save & migrate'),
      '#submit' => [
        '::submitForm',
        '_set_redirect_migration_confirm_page',
      ],
      '#button_type' => 'primary',
      '#weight' => -1,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    foreach ($form_state->getValue('bundle_list') as $bundle_id => $bundle_data) {
      if (!$bundle_data['deletion']) {
        continue;
      }

      if ($media_items = $this->entityTypeManager->getStorage('media')->loadByProperties(['bundle' => $bundle_id])) {
        $form_state->setError($form['bundle_list'][$bundle_id], $this->formatPlural(
          count($media_items),
          'Media type “%media_type_label” (<code>@media_type_id</code>) has a media item, thus this type cannot be deleted.',
          'Media type “%media_type_label” (<code>@media_type_id</code>) has @count media items, thus this type cannot be deleted.',
          [
            '%media_type_label' => $bundle_data['bundle_name'],
            '@media_type_id' => $bundle_id,
          ]
        ));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $data_to_save = [];
    foreach ($form_state->getValue('bundle_list') as $bundle_id => $bundle_data) {
      if ($bundle_data['deletion']) {
        $data_to_save['delete'][] = $bundle_id;
        continue;
      }

      $existing_bundle = $this->entityTypeManager->getStorage('media_type')->load($bundle_id);

      if ($bundle_data['bundle_name'] !== $existing_bundle->label()) {
        $data_to_save['update'][$bundle_id]['new_label'] = $bundle_data['bundle_name'];
      }

      // No "submitted vs. reality" comparison needed from this point here: legacy media types cannot have a value set
      // on their 'source_configuration.download_assets' config key, and target media types are rendered as plain text
      // with no option to change them provided.
      if (isset($bundle_data['fetch_method'])) {
        $data_to_save['update'][$bundle_id]['asset_download_enabled'] = ($bundle_data['fetch_method'] === 'sync');
      }

      if (isset($bundle_data['source_type'])) {
        $data_to_save['update'][$bundle_id]['source_plugin'] = $bundle_data['source_type'];
      }
    }

    // Store received input in the dedicated configuration.
    $this->configFactory->getEditable(self::MODULE_SETTINGS_NAME)->set('crud_operations', $data_to_save)->save();
  }

}
