<?php

declare(strict_types=1);

namespace Drupal\media_acquiadam\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the settings form to migrate Acquia DAM integration from a legacy to a more modern solution.
 */
class AcquiadamMigrationComplete extends ConfigFormBase {

  /**
   * The state storage service.
   *
   * @var \Drupal\Core\State\State
   */
  protected $stateStorage;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The migration service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Store module-wide used constant as class attribute.
   *
   * @var string
   */
  public const MODULE_SETTINGS_NAME = 'media_acquiadam.migration_preferences';

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->stateStorage = $container->get('state');
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->configFactory = $container->get('config.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_acquiadam_migration_complete';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      self::MODULE_SETTINGS_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $timestamp = $this->stateStorage->get('media_acquiadam.migration_process_finished');
    $this->messenger()->addWarning($this->t('The migration process was run earlier @duration-since ago at @date-time and re-running again automatically is not supported. However, if necessary, with some manual steps the site might be prepared for one more attempt. <a href=":url">Refer the documentation</a>', [
      '@duration-since' => $this->dateFormatter->formatTimeDiffSince($timestamp),
      '@date-time' => $this->dateFormatter->format($timestamp),
      // @todo update the URL to the actual documentation page.
      ':url' => '#',
    ]));
    unset($form['actions']);

    $migration_config = $this->configFactory->get(self::MODULE_SETTINGS_NAME)->get('crud_operations');

    // Get the migration information from state.
    $migrated_data = $this->stateStorage->get('media_acquiadam.migrated_data');

    // Create a table to display the number of media items for each media type.
    $form['table'] = [
      '#type' => 'table',
      '#header' => [
        'bundle_name' => $this->t('Media type'),
        'download_sync' => $this->t('Method for handling media'),
        'source_plugin' => $this->t('Target source type'),
        'migrated_asset_count' => $this->t('Number of assets migrated'),
      ],
      '#caption' => $this->t('This table shows the summary of media items migrated for each media type.'),
      '#empty' => $this->t('No assets migrated.'),
    ];

    // If no media items have been migrated,
    // display a message instead of the table.
    if (!empty($migrated_data)) {
      // Variable to get the display name of the Acquia DAM source plugin.
      $source_plugin_name = [
        'acquia_dam_asset:audio' => 'Acquia DAM: Audio',
        'acquia_dam_asset:documents' => 'Acquia DAM: Document',
        'acquia_dam_asset:generic' => 'Acquia DAM: Generic',
        'acquia_dam_asset:image' => 'Acquia DAM: Image',
        'acquia_dam_asset:pdf' => 'Acquia DAM: PDF',
        'acquia_dam_asset:spinset' => 'Acquia DAM: Spinset',
        'acquia_dam_asset:video' => 'Acquia DAM: Video',
      ];

      // Add the media types to the table.
      foreach ($migrated_data as $bundle_id => $info) {
        $form['table']['#rows'][] = [
          'bundle_name' => $info['bundle_name'],
          'download_sync' => $migration_config['update'][$bundle_id]['asset_download_enabled'] ? $this->t('Sync') : $this->t('Embed'),
          'source_plugin' => $source_plugin_name[$migration_config['update'][$bundle_id]['source_plugin']],
          'migrated_asset_count' => $info['migrated_assets_count'],
        ];
      }

    }

    return $form;
  }

}
