<?php

declare(strict_types=1);

namespace Drupal\media_acquiadam\Form;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Url;
use Drupal\media_acquiadam\Batch\MediaTypeProcessBatch;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form to confirm the Acquia DAM migration process.
 */
final class AcquiadamMigrationConfirm extends ConfirmFormBase {

  use LoggerChannelTrait;
  use MessengerTrait;

  /**
   * The migration service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The configuration data storing settings for the migration process.
   *
   * @var array|null
   */
  protected ?array $configData;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * AcquiadamMigrationConfirm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config entity storing settings for the migration process.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->configData = $config_factory->get(AcquiadamMigration::MODULE_SETTINGS_NAME)->get('crud_operations');
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_acquiadam_migration_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure that migration from legacy to modern Acquia DAM can be started?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('media_acquiadam.migration_config');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Bail out in the rare situation when the confirm page is invoked without any migration settings saved.
    if (!$this->configData) {
      $this->messenger()->addError($this->t('The Acquia DAM migration process cannot be initiated without having the preferences set first. <a href=":url">Configure migration</a>', [
        ':url' => Url::fromRoute('media_acquiadam.migration_config')->toString(),
      ]));

      return $form;
    }

    $form = parent::buildForm($form, $form_state);

    // Revert some page elements from parent method.
    unset($form['#title']);
    unset($form['description']);

    // Set a less emphasised title instead.
    $form['introductional_guide'][] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $this->getQuestion(),
    ];

    $form['introductional_guide'][] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => parent::getDescription(),
    ];

    if (isset($this->configData['delete']) && $this->configData['delete']) {
      $bundle_to_delete = [];
      foreach ($this->configData['delete'] as $bundle) {
        $bundle_to_delete[] = $this->entityTypeManager->getStorage('media_type')->load($bundle)->label();;
      }
      $form['media_bundle_delete_info'][] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => '<h4>Following media types will be deleted: </h4>' . implode(", ", $bundle_to_delete),
      ];
    }

    if (isset($this->configData['update']) && $this->configData['update']) {
      $form['media_bundle_update_info'] = [
        '#type' => 'table',
        '#header' => [
          $this->t('Bundle Name'),
          $this->t('Method to handle media files'),
          $this->t('Target source type'),
          $this->t('Number of Media Items'),
        ],
        '#caption' => $this->t('<h4>This table lists the media bundles that will be updated during the migration process.</h4>'),
        '#rows' => [],
      ];
      foreach ($this->configData['update'] as $bundle_id => $bundle_data) {
        $media_label = $this->entityTypeManager->getStorage('media_type')->load($bundle_id)->label();
        $media_items = $this->entityTypeManager->getStorage('media')->loadByProperties(['bundle' => $bundle_id]);
        $source_plugin = \Drupal::service('plugin.manager.media.source')->createInstance($bundle_data['source_plugin']);
        $source_plugin_label = $source_plugin->getPluginDefinition()['label'];
        $form['media_bundle_update_info']['#rows'][] = [
          $media_label,
          $bundle_data['asset_download_enabled'] === true ? $this->t('Download and Sync') : $this->t('Embed'),
          $source_plugin_label,
          count($media_items),
        ];
      }
    }


    $form['actions']['cancel']['#attributes'] = [
      'class' => ['button', 'button--danger'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $batch_builder = new BatchBuilder();
    $batch_builder
      ->setTitle($this->t("Acquia DAM migration is in progress\u{2026}"))
      ->setInitMessage($this->t("Preparing for processing media types and items\u{2026}"))
      ->setErrorMessage($this->t('An error happened during Acquia DAM migration while processing media types and items.'))
      ->setFinishCallback([MediaTypeProcessBatch::class, 'finish']);

    foreach (['update', 'delete'] as $operation) {
      if (!isset($this->configData[$operation])) {
        continue;
      }

      foreach ($this->configData[$operation] as $key => $value) {
        $media_type_id = ($operation === 'update') ? $key : $value;
        $batch_builder->addOperation([MediaTypeProcessBatch::class, 'process'], [$operation, $media_type_id, $this->configData[$operation]]);
      }
    }

    batch_set($batch_builder->toArray());
    $form_state->setRedirectUrl(Url::fromRoute('entity.media_type.collection'));
  }

}
