<?php

declare(strict_types=1);

namespace Drupal\Tests\media_acquiadam\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the build of the migration form by checking its render array in various situations.
 *
 * @group media_acquiadam
 */
class AcquiadamMigrationFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'acquia_dam',
    'media_acquiadam',
    'media_acquiadam_example'
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  // Remove when fallback_formatter 8.x-1.5 is released.
  // phpcs:ignore DrupalPractice.Objects.StrictSchemaDisabled.StrictConfigSchema
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalLogin($this->createUser(['administer site configuration']));
  }

  /**
   * Tests the build of the migration form in normal conditions.
   */
  public function testMigration() {
    $this->drupalGet('/admin/config/acquia-dam/migration');
    $page = $this->getSession()->getPage();
    $assert_session = $this->assertSession();
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Migrate from legacy to modern Acquia DAM');
    $assert_session->pageTextContains("This page lists all the media types that are using any source plugin provided by either the Media: Acquia DAM (media_acquiadam) or the Acquia DAM (acquia_dam) modules. In most cases, this results in twelve rows: five default media types from the prior and seven from the latter module. However, any other manually created, custom media types using such source plugins are also listed. Media types of Drupal core's Media API or defined by any other contrib module do not appear here.");
    $assert_session->pageTextContains("The first segment of the list displays the media types provided by the “legacy” module. Their human label can be renamed and their machine name is displayed only for informational purposes. If any media item exists on the site of a given type, then their deletion is disallowed. If a legacy media type does not contain any media item, then their deletion is recommended.");
    $assert_session->pageTextContains("If a legacy media type is being kept (most probably because it contains media items), then choosing a file-handling method is mandatory. If the legacy media type's name suggests that it can contain “Audio” or “Generic” type of assets from Widen DAM, then the Sync method is suggested to apply. If you know about any other manually created, custom media types containing such assets too, then please ensure that you also opt for the Sync method for them as well.");
    $assert_session->pageTextContains("Deciding about the target source plugin is also mandatory for all legacy media types not being deleted. The drop-down field lists source plugins provided by the modern module only. Some automated guesswork is also implemented here to offer suggestions but always pay attention to opting for the most appropriate source plugin for each legacy media type. If no match is found, then choose the “Generic” one.");
    $assert_session->pageTextContains("The lower half of the table lists the media types provided by the modern module. Mostly the same terms apply to them except their file-handling and source plugin cannot be changed anymore.");
    // Assert that migration table exists.
    $assert_session->elementExists('css', 'table.responsive-enabled');
    // Assert that  migration table has header exist.
    $assert_session->pageTextContains('Media type label and name');
    $assert_session->pageTextContains('Number of media items');
    $assert_session->pageTextContains('Method to handle media files');
    $assert_session->pageTextContains('Target source type');
    // Submit the form and ensure that we see the expected message(s).
    $page->pressButton('Save');
    $assert_session->pageTextContains('The configuration options have been saved.');
    // Validate the redirect.
    $assert_session->addressEquals('/admin/config/acquia-dam/migration');
    $assert_session->statusCodeEquals(200);

    // Submit the form and ensure that we see the expected message(s).
    $page->pressButton('Save & migrate');
    $assert_session->pageTextContains('The configuration options have been saved.');
    // Validate the redirect.
    $assert_session->addressEquals('/admin/config/acquia-dam/migration/confirmation');
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Are you sure that migration from legacy to modern Acquia DAM can be started?');
    $assert_session->pageTextContains('This action cannot be undone.');
  }

  /**
   * Tests the form build when migration has been run earlier already.
   */
  public function testMigrationComplete() {
    // Mock the situation: set a past timestamp in State storage.
    \Drupal::state()->set('media_acquiadam.migration_process_finished', '1713545861');

    $this->drupalGet('/admin/config/acquia-dam/migration');
    $assert_session = $this->assertSession();
    $assert_session->statusCodeEquals(200);
    // Assert that the warning message is displayed and get its text.
    $warning_message = $this->getSession()->getPage()->find('css', 'div[role="contentinfo"]')->getText();
    $this->assertNotEmpty($warning_message, 'Warning message is not empty.');
    $this->assertMatchesRegularExpression('/^Warning message The migration process was run earlier .+ ago at .+ and re-running again automatically is not supported. However, if necessary, with some manual steps the site might be prepared for one more attempt\./', $warning_message);
    $assert_session->pageTextContains('Migration from legacy to modern Acquia DAM completed');
    $assert_session->pageTextContains('This table shows the summary of media items migrated for each media type.');
  }

}
