<?php

declare(strict_types=1);

namespace Drupal\Tests\media_acquiadam\Kernel;

use Drupal\Core\Config\Config;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormState;
use Drupal\media_acquiadam\Form\AcquiadamMigration;
use Drupal\Tests\media_acquiadam\Traits\AcquiadamAssetDataTrait;

/**
 * Tests the build of the migration form by checking its render array in various situations.
 *
 * @group media_acquiadam
 */
class AcquiadamMigrationFormBuildTest extends AcquiadamKernelTestBase {

  use AcquiadamAssetDataTrait;

  // Remove when fallback_formatter 8.x-1.5 is released.
  // phpcs:ignore DrupalPractice.Objects.StrictSchemaDisabled.StrictConfigSchema
  protected $strictConfigSchema = FALSE;

  /**
   * @var array
   */
  protected static $modules = [
    'acquia_dam',
    'dblog',
    'fallback_formatter',
    'file',
    'image',
    'media',
    'media_acquiadam',
    'media_acquiadam_test',
    'media_library',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->asset = $this->getAssetData();

    // Create file with same name as asset file to make sure asset file
    // replacement happens as expected.
    $dir_path = 'public://acquiadam/';
    $contents = 'test';
    $this->container->get('file_system')->prepareDirectory($dir_path, FileSystemInterface::CREATE_DIRECTORY);
    $this->container->get('file_system')->saveData($contents, $dir_path . $this->asset->filename);

    $this->testClient->addAsset($this->asset);
    $this->createMedia($this->asset->id, 'non_empty_media_type');

    // Add necessary DB tables in extra.
    $this->installSchema('dblog', ['watchdog']);
    $this->installSchema('user', ['users_data']);

    // Store some dummy migration settings.
    // @see \Drupal\KernelTests\Core\Config\ConfigEventsTest::testConfigEvents()
    $config_entity = new Config(
      'media_acquiadam.migration_preferences',
      $this->container->get('config.storage'),
      $this->container->get('event_dispatcher'),
      $this->container->get('config.typed')
    );
    $config_entity->set('crud_operations', [
      'update' => [
        'acquia_dam_other_asset' => [
          'new_label' => 'Other test media type renamed',
          'asset_download_enabled' => TRUE,
          'source_plugin' => 'acquia_dam_asset:video',
        ],
        'non_existent_media_type_to_update' => [
          'new_label' => 'Does not exist',
          'source_plugin' => 'theres_no_such_source',
        ],
      ],
      'delete' => ['non_existent_media_type_to_delete'],
    ]);
    $this->assertSame([], \Drupal::state()->get('config_events_test.event', []), 'No events fired by creating a new configuration object');
    $config_entity->save();
  }

  /**
   * Tests the build of the migration form in normal conditions.
   */
  public function testMigrationFormBuild() {
    // No mocking as this is the regular situation.
    $form_array = $this->_prepareMigrationForm();
    $this->assertIsArray($form_array);
    $this->assertNotEmpty($form_array);
    $this->assertCount(5, $form_array['introductional_guide']['children'], 'The migration form must start with five paragraphs of guidance text.');

    $this->assertArrayHasKey('acquia_dam_asset', $form_array['bundle_list']);
    $this->assertEquals('Suggestion: <em>Sync</em>', $form_array['bundle_list']['acquia_dam_asset']['fetch_method']['#description']->getUntranslatedString(), 'Sync fetch method should be suggested for the given media type.');
    $this->assertEquals('acquia_dam_asset:generic', $form_array['bundle_list']['acquia_dam_asset']['source_type']['#default_value'], 'Target source plugin should be pre-selected for the given media type.');

    $this->assertArrayHasKey('acquia_dam_other_asset', $form_array['bundle_list']);
    $this->assertEquals('Other test media type renamed', $form_array['bundle_list']['acquia_dam_other_asset']['bundle_name']['#value'], 'New bundle name should be filled in from config for the given media type.');
    $this->assertEquals('sync', $form_array['bundle_list']['acquia_dam_other_asset']['fetch_method']['#default_value'], 'Target source plugin should be pre-selected for the given media type.');
    $this->assertEquals('acquia_dam_asset:video', $form_array['bundle_list']['acquia_dam_other_asset']['source_type']['#default_value'], 'Target source plugin should be pre-selected for the given media type.');

    $this->assertArrayHasKey('non_empty_media_type', $form_array['bundle_list']);
    $this->assertEquals(1, $form_array['bundle_list']['non_empty_media_type']['item_count']['#value'], 'The given media type should count one media item.');
    $this->assertTrue($form_array['bundle_list']['non_empty_media_type']['deletion']['#disabled'], 'Delete option of the given media type should be disabled.');

    $this->assertArrayHasKey('empty_media_type', $form_array['bundle_list']);
    $this->assertEquals('None', $form_array['bundle_list']['empty_media_type']['item_count']['#markup']->getUntranslatedString(), 'The given media type should count zero media item.');
    $this->assertTrue($form_array['bundle_list']['empty_media_type']['deletion']['#default_value'], 'Delete option of the given media type should be marked by default.');

    $this->assertArrayNotHasKey('non_existent_media_type_to_update', $form_array['bundle_list']);
    $this->assertArrayNotHasKey('non_existent_media_type_to_delete', $form_array['bundle_list']);
  }

  /**
   * Tests the config form build when the 'Acquia DAM' module is not present.
   */
  public function testMigrationFormBuildMissingModernModule() {
    // Mock the situation: remove the 'Acquia DAM' module.
    \Drupal::service('module_installer')->uninstall(['acquia_dam']);

    $form_array = $this->_prepareMigrationForm();
    $this->assertIsArray($form_array);
    $this->assertNotEmpty($form_array);
    $error_messages = \Drupal::service('messenger')->messagesByType('error');
    $this->assertNotEmpty($error_messages);
    $this->assertMatchesRegularExpression('/^The migration process requires that both the source <em>Media: Acquia DAM<\/em> \(<code>media_acquiadam<\/code>\) and the target <em>Acquia DAM<\/em> \(<code>acquia_dam<\/code>\) modules be installed on the website\./', $error_messages[0]->__toString());
  }

  /**
   * Tests the config form build when no DAM-related media types exist at all.
   */
  public function testMigrationFormBuildNoMediaTypesExist() {
    // Mock the situation: delete all media types.
    $entity_type_storage = $this->entityTypeManager->getStorage('media_type');
    $entity_type_storage->delete($entity_type_storage->loadMultiple());
    $this->assertEmpty($entity_type_storage->loadMultiple());

    $form_array = $this->_prepareMigrationForm();
    $this->assertIsArray($form_array);
    $this->assertNotEmpty($form_array);
    $error_messages = \Drupal::service('messenger')->messagesByType('error');
    $this->assertNotEmpty($error_messages);
    $this->assertMatchesRegularExpression('/^No DAM-related media types exist on the site. The migration process handles only media types which are based on the source plugins provided by either the <em>Media: Acquia DAM<\/em> \(<code>media_acquiadam<\/code>\) or the <em>Acquia DAM<\/em> \(<code>acquia_dam<\/code>\) modules\./', $error_messages[0]->__toString());
  }

  /**
   * Internal helper method to return a migration config form.
   *
   * @return array
   *   The render array generated by the form controller.
   */
  protected function _prepareMigrationForm(): array {
    $migrate_form = AcquiadamMigration::create($this->container);

    $form_state = new FormState();
    $form_state->setFormObject($migrate_form);
    $form_array = [];
    return $migrate_form->buildForm($form_array, $form_state);
  }

}
