<?php

declare(strict_types=1);

namespace Drupal\Tests\media_acquiadam\Kernel;

use Drupal\Core\Config\Config;
use Drupal\Core\Form\FormState;
use Drupal\media_acquiadam\Batch\MediaTypeProcessBatch;
use Drupal\media_acquiadam\Form\AcquiadamMigrationConfirm;
use Drupal\Tests\media_acquiadam\Traits\AcquiadamAssetDataTrait;

/**
 * Test batch processing of Acquia DAM migration preferences stored in config.
 *
 * @group media_acquiadam
 */
class AcquiadamMigrationProcessTest extends AcquiadamKernelTestBase {

  use AcquiadamAssetDataTrait;

  // Remove when fallback_formatter 8.x-1.5 is released.
  // phpcs:ignore DrupalPractice.Objects.StrictSchemaDisabled.StrictConfigSchema
  protected $strictConfigSchema = FALSE;

  /**
   * Add 'dblog' in extra to the parent's modules.
   *
   * @var array
   */
  protected static $modules = [
    'acquia_dam',
    'dblog',
    'fallback_formatter',
    'file',
    'image',
    'media',
    'media_acquiadam',
    'media_acquiadam_test',
    'entity_browser',
    'media_library',
    'node',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Install config of media_library module.
    $this->installConfig(['media_library']);

    // Add 'dblog' in extra to the parent's modules.
    $this->installSchema('dblog', ['watchdog']);

    // Store some dummy migration settings to test processing with.
    // @see \Drupal\KernelTests\Core\Config\ConfigEventsTest::testConfigEvents()
    $config_entity = new Config(
      'media_acquiadam.migration_preferences',
      $this->container->get('config.storage'),
      $this->container->get('event_dispatcher'),
      $this->container->get('config.typed')
    );
    $config_entity->set('crud_operations', [
      'update' => [
        // This bundle contains 1 piece of media item.
        'acquia_dam_asset' => [
          'new_label' => 'Test media type renamed',
          'asset_download_enabled' => TRUE,
          'source_plugin' => 'acquia_dam_asset:image',
        ],
        'acquia_dam_other_asset' => [
          'new_label' => 'Other test media type also renamed',
          'asset_download_enabled' => FALSE,
          'source_plugin' => 'acquia_dam_asset:video',
        ],
        'non_existent_media_type_to_update' => [
          'new_label' => 'Does not exist',
          'source_plugin' => 'theres_no_such_source',
        ],
      ],
      'delete' => ['non_existent_media_type_to_delete'],
    ]);
    $this->assertSame([], \Drupal::state()->get('config_events_test.event', []), 'No events fired by creating a new configuration object');
    $config_entity->save();
  }

  /**
   * Tests processing the config entity storing migration settings.
   */
  public function testConfigEntityProcess() {
    $confirm_form = new AcquiadamMigrationConfirm(
      $this->container->get('config.factory'),
      $this->container->get('entity_type.manager'),
    );

    $form_state = new FormState();
    $form_state->setFormObject($confirm_form);
    $form_array = [];
    $confirm_form->submitForm($form_array, $form_state);
    $batch =& batch_get();
    $batch['progressive'] = FALSE;
    batch_process();

    $media_type_storage = $this->entityTypeManager->getStorage('media_type');
    $existing_media_type = $media_type_storage->load('acquia_dam_asset');
    $this->assertSame('Test media type renamed', $existing_media_type->label());
    $this->assertSame('acquia_dam_asset:image', $existing_media_type->get('source'));
    $existing_media_type_source_configuration = $existing_media_type->get('source_configuration');
    $this->assertTrue($existing_media_type_source_configuration['download_assets']);

    $existing_media_type = $media_type_storage->load('acquia_dam_other_asset');
    $this->assertSame('Other test media type also renamed', $existing_media_type->label());
    $this->assertSame('acquia_dam_asset:video', $existing_media_type->get('source'));
    $existing_media_type_source_configuration = $existing_media_type->get('source_configuration');
    $this->assertFalse($existing_media_type_source_configuration['download_assets']);
  }

  /**
   *  Tests the entityBrowserToMediaLibrary method.
   */
  public function testEntityBrowserToMediaLibrary() {
    // Create a new content type.
    $content_type = $this->createContentType();

    // Create field storage for DAM media.
    $field_storage = $this->createFieldStorage('node', 'entity_reference', 'field_dam_media', ['target_type' => 'media']);

    // Add a field to the content type for DAM media.
    $field_instance_dam = $this->addField($content_type->id(), $field_storage, [
      'handler' => 'default:media',
      'handler_settings' => [
        'target_bundles' => [
          'acquia_dam_asset' => 'acquia_dam_asset'
        ],
      ],
    ]);

    // Create field storage for local media.
    $field_storage = $this->createFieldStorage('node', 'entity_reference', 'field_local_media', ['target_type' => 'media']);

    // Add a field to the content type for local media.
    $field_instance_local = $this->addField($content_type->id(), $field_storage, [
      'handler' => 'default:media',
      'handler_settings' => [
        'target_bundles' => [
          'image' => 'image'
        ],
      ],
    ]);

    // Create default form display for the content type.
    $default_form_display = $this->entityTypeManager->getStorage('entity_form_display')->create([
      'targetEntityType' => 'node',
      'bundle' => $content_type->id(),
      'mode' => 'default',
      'status' => TRUE,
    ]);

    // Add the DAM media and local media fields to the form display.
    foreach ([$field_instance_dam->get('field_name'), $field_instance_local->get('field_name')] as $field_name) {
      $default_form_display->setComponent($field_name, [
        'type' => 'entity_browser_entity_reference',
        'label' => 'hidden',
        'settings' => [
          'entity_browser' => 'media',
          'field_widget_display' => 'label',
          'open' => FALSE,
          'selection_mode' => 'selection_append',
        ],
        'third_party_settings' => [],
        'region' => 'content',
      ]);
    }
    $default_form_display->save();

    // Invoke method to update form display to use the media library widget.
    MediaTypeProcessBatch::entityBrowserToMediaLibrary();

    // Load back updated form display for the content type.
    $updated_form_display = $this->entityTypeManager->getStorage('entity_form_display')->load('node.' . $content_type->id() . '.default');

    $actual_component = $default_form_display->getComponent($field_instance_dam->get('field_name'));

    // Update the component type and settings that should be changed.
    $actual_component['type'] = 'media_library_widget';
    $actual_component['settings'] = [
      'media_types' => $field_instance_dam->getSetting('handler_settings')['target_bundles'],
    ];

    // Assert that the DAM media field has been updated.
    $this->assertEquals($actual_component, $updated_form_display->getComponent($field_instance_dam->get('field_name')));

    // Verify that the local media field remains unchanged.
    $this->assertEquals($default_form_display->getComponent($field_instance_local->get('field_name')), $updated_form_display->getComponent($field_instance_local->get('field_name')));
  }

}
